function varargout=configure(varargin)
% FUNCTION fc_oogmsh.configure()
%   Configures the toolbox/package by setting gmsh path, meshes default 
%   directory and .geo files default directory.
%   Theses informations will be stored in ... file.
%
% <COPYRIGHT>
  [conffile,isFileExists]=fc_oogmsh.getLocalConfFile();
  if isFileExists
    run(conffile);
  else
    fullname=mfilename('fullpath');
    I=strfind(fullname,filesep);
    Path=fullname(1:(I(end-1)-1));
    gmsh_bin=getDefaultGmshBin();
    mesh_dir=[Path,filesep,'meshes'];
    geo_dir=[Path,filesep,'geodir'];
    fc_tools_dir=''; % empty if pkg or toolbox in path
  end
  p = inputParser;
  p.addParamValue('gmsh_bin',gmsh_bin,@ischar);
  p.addParamValue('mesh_dir',mesh_dir,@ischar);
  p.addParamValue('geo_dir',geo_dir,@ischar);
  p.addParamValue('fc_tools_dir',fc_tools_dir,@ischar);
  %p.addParamValue('force',false,@islogical); 
  p.parse(varargin{:});
  R=p.Results;
  if length(R.gmsh_bin)==0,R.gmsh_bin=getDefaultGmshBin();end
  
  if ~isdir(R.mesh_dir)
    try
      mkdir(R.mesh_dir);
    catch
      error('Unable to create directory: %s\n',R.mesh_dir)
    end
  end
  
  %if ~isdir(R.mesh_dir), error('Not a directory :\n''mesh_dir''=%s\n', R.mesh_dir); end
  if ~isdir(R.geo_dir), error('Not a directory :\n''geo_dir''=%s\n', R.geo_dir); end
  %if exist(R.gmsh_bin)~=2, error('Not a file :\n''gmsh_bin''=%s\n', R.gmsh_bin); end
  if ismac()
    if exist( R.gmsh_bin)~=2
      R.gmsh_bin=fullfile(R.gmsh_bin,'Contents','MacOS','gmsh');
    end
  end
  if exist( R.gmsh_bin)~=2
    fprintf('     -> Select gmsh binary ...\n');
    if ispc()
      [gmsh_bin, gmsh_dir]=uigetfile('*.exe','Select GMSH binary',R.gmsh_bin);
    else
      [gmsh_bin, gmsh_dir]=uigetfile('*','Select GMSH binary',R.gmsh_bin);
      if ismac() && strcmp(gmsh_bin,'Gmsh.app')
        gmsh_dir=fullfile(gmsh_dir,gmsh_bin,'Contents','MacOS');
        gmsh_bin='gmsh';
      end
    end
    R.gmsh_bin=fullfile(gmsh_dir,gmsh_bin);
    if exist(R.gmsh_bin)~=2
      warning('Failed to load GMSH binary \n  -> %s\n',R.gmsh_bin)
    end
  %else
  %  [gmsh_dir,gmsh_bin,gmsh_ext]=fileparts(R.gmsh_bin);
  %  if ~isempty(gmsh_ext),gmsh_bin=[gmsh_bin,'.',gmsh_ext];end
  end
  fprintf('     -> Using GMSH binary : %s\n',R.gmsh_bin);
  
  fc_tools_dir=get_tbxpath('tools','oogmsh',R.fc_tools_dir);
  
  fprintf('Write in %s ...\n',conffile);
  fid=fopen(conffile,'w');
  if (fid==0), error('Unable to open file :\n %s\n',conffile);end
  fprintf(fid,'%% Automaticaly generated with fc_oogmsh.configure()\n');
  fprintf(fid,'gmsh_bin=''%s'';\n',R.gmsh_bin);
  fprintf(fid,'mesh_dir=''%s'';\n',R.mesh_dir);
  fprintf(fid,'geo_dir=''%s'';\n',R.geo_dir);
  fprintf(fid,'fc_tools_dir=''%s'';\n',fc_tools_dir);
  fclose(fid);
  fprintf('  -> done\n');
  %eval(sprintf('clear %s', conffile))
  if nargout==1,varargout{1}=conffile;end
end

function default_gmsh_bin=getDefaultGmshBin()
  if ispc()
    default_gmsh_bin=fullfile([getenv('HOMEDRIVE') getenv('HOMEPATH')],'Softwares','GMSH','gmsh.exe');
  elseif ismac()
    default_gmsh_bin=fullfile(getenv('HOME'),'GMSH','Gmsh.app','Contents','MacOS','gmsh');
  else
    default_gmsh_bin=fullfile(getenv('HOME'),'bin','gmsh');
  end
end

function tbxpath=get_tbxpath(tbxname,tbxfrom,givenpath) % tbxname is the toolbox to 'include' in the toolbox <tbxfrom>
  tbxpath='';
  if ~isempty(givenpath) 
    if ~isdir(givenpath) 
      fprintf('[fc-%s] The given path does not exists:\n  %s\n',tbxfrom,tbxname,givenpath)
      fprintf('[fc-%s] Use fc_%s.configure(''fc_%s_dir'',<DIR>) to correct this issue\n\n',tbxfrom,tbxfrom,tbxname);
      error(sprint('fc-%s::configure',tbxfrom))
    end
    tbxpath=givenpath;
    addpath(tbxpath);rehash path;
    failed=false;
    try % check if the toolbox is in current Matlab path
      eval(sprintf('fc_%s.init()',tbxname))
    catch
      fprintf('[fc-%s] Unable to load the fc-%s toolbox/package in given path:\n  %s\n',tbxfrom,tbxname,tbxpath)
      error(sprintf('fc-%s::configure',tbxfrom))
    end   
  end
  
  failed=false;
  try % check if the toolbox is in current Matlab path
    eval(sprintf('fc_%s.init()',tbxname))
  catch
    fprintf('[fc-%s] Unable to load the fc-%s toolbox/package in current path\n',tbxfrom,tbxname)
    failed=true;
  end
  if ~failed, return;end
  fullname=mfilename('fullpath');
  I=strfind(fullname,filesep);
  Path=fullname(1:(I(end-2)-1));
  tbxpath=[Path,filesep,'fc-',tbxname];
  if isdir(tbxpath), addpath(tbxpath);rehash path;end
  try % check if the toolbox is in new current Matlab path
    eval(sprintf('fc_%s.init()',tbxname))
  catch
    fprintf('[fc-%s] Unable to load the fc-%s toolbox/package in guess path\n  %s\n',tbxfrom,tbxname,tbxpath)
    fprintf('[fc-%s] Use fc_%s.configure(''fc_%s_dir'',<DIR>) to correct this issue\n\n',tbxfrom,tbxfrom,tbxname);
    error(sprintf('fc-%s::configure',tbxfrom))
  end  
end
